@@ -42,6 +42,7 @@
 #undef NDEBUG /* Keep test strong even for nbdkit built without assertions */
 #include <assert.h>
 
+#include "array-size.h"
 #include "bench.h"
 #include "const-string-vector.h"
 #include "nbdkit-string.h"
@@ -50,11 +51,46 @@
 
 #define APPENDS 1000000
 
+DEFINE_VECTOR_TYPE (int_vector, int);
 DEFINE_VECTOR_TYPE (int64_vector, int64_t);
 DEFINE_VECTOR_TYPE (uint32_vector, uint32_t);
 
+static void
+test_int_vector (void)
+{
+  int_vector v = empty_vector;
+  int data[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };
+  int r;
+  size_t i;
+
+  r = int_vector_append_array (&v, data, ARRAY_SIZE (data));
+  assert (r == 0);
+
+  assert (v.len == 10);
+  for (i = 0; i < 10; ++i)
+    assert (v.ptr[i] == i);
+
+  int_vector_remove_range (&v, 5, 3); /* remove elements [3..7] */
+
+  assert (v.len == 5);
+  assert (v.ptr[0] == 0);
+  assert (v.ptr[1] == 1);
+  assert (v.ptr[2] == 2);
+  assert (v.ptr[3] == 8);
+  assert (v.ptr[4] == 9);
+
+  r = int_vector_insert_array (&v, &data[3], 5, 3);
+  assert (r == 0);
+
+  assert (v.len == 10);
+  for (i = 0; i < 10; ++i)
+    assert (v.ptr[i] == i);
+
+  int_vector_reset (&v);
+}
+
 static int
-compare (const int64_t *a, const int64_t *b)
+compare_int64 (const int64_t *a, const int64_t *b)
 {
   return (*a > *b) - (*a < *b);
 }
@@ -73,7 +109,7 @@ test_int64_vector (void)
 
   for (i = 0; i < 10; ++i)
     assert (v.ptr[i] == 9 - i);
-  int64_vector_sort (&v, compare);
+  int64_vector_sort (&v, compare_int64);
   for (i = 0; i < 10; ++i)
     assert (v.ptr[i] == i);
 
@@ -82,13 +118,13 @@ test_int64_vector (void)
   assert (v.ptr[1] == 2);
 
   tmp = 10;
-  p = int64_vector_search (&v, &tmp, (void *)compare);
+  p = int64_vector_search (&v, &tmp, (void*) compare_int64);
   assert (p == NULL);
   tmp = 8;
-  p = int64_vector_search (&v, &tmp, (void *)compare);
+  p = int64_vector_search (&v, &tmp, (void*) compare_int64);
   assert (p == &v.ptr[7]);
 
-  free (v.ptr);
+  int64_vector_reset (&v);
 }
 
 static void
@@ -124,7 +160,7 @@ test_string (void)
 
   assert (strcmp (s.ptr, "hello world") == 0);
   assert (s.len == 12); /* hello + space + world + \0 */
-  free (s.ptr);
+  string_reset (&s);
 }
 
 /* Same as above, but using string_reserve_exactly. */
@@ -161,7 +197,7 @@ test_string_exactly (void)
 
   assert (strcmp (s.ptr, "hello world") == 0);
   assert (s.len == 12); /* hello + space + world + \0 */
-  free (s.ptr);
+  string_reset (&s);
 }
 
 static void
@@ -232,6 +268,45 @@ test_overflow (void)
   assert (errno == ENOMEM);
 }
 
+/* Test vector_uniq function. */
+static int
+compare_int (const int *a, const int *b)
+{
+  return (*a > *b) - (*a < *b);
+}
+
+static void
+test_uniq (void)
+{
+  int_vector v = empty_vector;
+  int r;
+
+  int data1[] = { 1, 2, 2, 2, 3, 1, 1, 1, 4, 4 };
+  r = int_vector_append_array (&v, data1, ARRAY_SIZE (data1));
+  assert (r == 0);
+  int_vector_uniq (&v, compare_int);
+  assert (v.len == 5);
+  assert (v.ptr[0] == 1);
+  assert (v.ptr[1] == 2);
+  assert (v.ptr[2] == 3);
+  assert (v.ptr[3] == 1);
+  assert (v.ptr[4] == 4);
+  int_vector_reset (&v);
+
+  int data2[] = { 1, 2, 1, 2, 2, 2, 3, 1, 1, 1 };
+  r = int_vector_append_array (&v, data2, ARRAY_SIZE (data2));
+  assert (r == 0);
+  int_vector_uniq (&v, compare_int);
+  assert (v.len == 6);
+  assert (v.ptr[0] == 1);
+  assert (v.ptr[1] == 2);
+  assert (v.ptr[2] == 1);
+  assert (v.ptr[3] == 2);
+  assert (v.ptr[4] == 3);
+  assert (v.ptr[5] == 1);
+  int_vector_reset (&v);
+}
+
 static void
 bench_reserve (void)
 {
@@ -287,12 +362,14 @@ main (int argc, char *argv[])
 
   if (!bench) {
     /* Do normal tests. */
+    test_int_vector ();
     test_int64_vector ();
     test_string ();
     test_string_exactly ();
     test_string_vector ();
     test_const_string_vector ();
     test_overflow ();
+    test_uniq ();
   }
 
   else {
