@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbd client library in userspace
-# Copyright (C) 2019 Red Hat Inc.
+# Copyright Red Hat
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -16,18 +16,19 @@
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-# Test that nbddump --version looks sane.
+# Test that --version looks sane.
 
-fail=0
-output=$($VG nbddump --version)
-if [ $? != 0 ]; then
-    echo "$0: unexpected exit status"
-    fail=1
-fi
-if [ "$output" != "nbddump $EXPECTED_VERSION
-libnbd $EXPECTED_VERSION" ]; then
-    echo "$0: unexpected output"
-    fail=1
-fi
-echo "$output"
-exit $fail
+. ../tests/functions.sh
+set -e
+set -x
+
+tool=nbddump
+
+output=test-$tool.out
+cleanup_fn rm -f $output
+
+$VG $tool --version > $output
+cat $output
+
+grep "$tool $EXPECTED_VERSION" $output
+grep "libnbd $EXPECTED_VERSION" $output
