@@ -33,8 +33,7 @@ let generate_python_methods_h () =
   pr "\n";
   pr "#include <assert.h>\n";
   pr "\n";
-  pr "\
-extern char **nbd_internal_py_get_string_list (PyObject *);
+  pr {|extern char **nbd_internal_py_get_string_list (PyObject *);
 extern void nbd_internal_py_free_string_list (char **);
 extern int nbd_internal_py_get_sockaddr (PyObject *,
     struct sockaddr_storage *, socklen_t *);
@@ -49,7 +48,7 @@ get_handle (PyObject *obj)
 {
   assert (obj);
   assert (obj != Py_None);
-  return PyCapsule_GetPointer(obj, \"nbd_handle\");
+  return PyCapsule_GetPointer(obj, "nbd_handle");
 }
 
 /* nbd.Error exception. */
@@ -58,7 +57,7 @@ extern PyObject *nbd_internal_py_Error;
 static inline void
 raise_exception ()
 {
-  PyObject *args = Py_BuildValue (\"si\", nbd_get_error (), nbd_get_errno ());
+  PyObject *args = Py_BuildValue ("si", nbd_get_error (), nbd_get_errno ());
 
   if (args != NULL) {
     PyErr_SetObject (nbd_internal_py_Error, args);
@@ -66,7 +65,7 @@ raise_exception ()
   }
 }
 
-";
+|};
 
   List.iter (
     fun name ->
@@ -113,11 +112,10 @@ let generate_python_libnbdmod_c () =
   pr "  { NULL, NULL, 0, NULL }\n";
   pr "};\n";
   pr "\n";
-  pr "\
-static struct PyModuleDef moduledef = {
+  pr {|static struct PyModuleDef moduledef = {
   PyModuleDef_HEAD_INIT,
-  \"libnbdmod\",           /* m_name */
-  \"libnbd module\",       /* m_doc */
+  "libnbdmod",           /* m_name */
+  "libnbd module",       /* m_doc */
   -1,                    /* m_size */
   methods,               /* m_methods */
   NULL,                  /* m_reload */
@@ -140,8 +138,8 @@ PyInit_libnbdmod (void)
   if (mod == NULL)
     return NULL;
 
-  nbd_internal_py_Error = PyErr_NewException (\"nbd.Error\", NULL, NULL);
-  if (PyModule_AddObject (mod, \"Error\", nbd_internal_py_Error) < 0) {
+  nbd_internal_py_Error = PyErr_NewException ("nbd.Error", NULL, NULL);
+  if (PyModule_AddObject (mod, "Error", nbd_internal_py_Error) < 0) {
     Py_XDECREF (nbd_internal_py_Error);
     Py_DECREF (mod);
     return NULL;
@@ -149,7 +147,7 @@ PyInit_libnbdmod (void)
 
   return mod;
 }
-"
+|}
 
 (* Functions with a Closure parameter are special because we
  * have to generate wrapper functions which translate the
@@ -648,19 +646,18 @@ let generate_python_methods_c () =
       print_python_binding name fn
   ) handle_calls
 
-let py_fn_rex = Str.regexp "L<nbd_\\([a-z0-9_]+\\)(3)>"
-let py_const_rex = Str.regexp "C<LIBNBD_\\([A-Z0-9_]+\\)>"
+let py_fn_rex = Str.regexp {|L<nbd_\([a-z0-9_]+\)(3)>|}
+let py_const_rex = Str.regexp {|C<LIBNBD_\([A-Z0-9_]+\)>|}
 
 let generate_python_nbd_py () =
   generate_header HashStyle;
 
-  pr "\
-'''
+  pr {|'''
 Python bindings for libnbd
 
 import nbd
 h = nbd.NBD()
-h.connect_tcp(\"localhost\", \"nbd\")
+h.connect_tcp("localhost", "nbd")
 buf = h.pread(512, 0)
 
 Read the libnbd(3) man page to find out how to use the API.
@@ -703,9 +700,9 @@ Error.errnum = property(lambda self: self.args[1])
 
 def _str(self):
     if self.errno:
-        return \"%%s (%%s)\" %% (self.string, self.errno)
+        return "%%s (%%s)" %% (self.string, self.errno)
     else:
-        return \"%%s\" %% self.string
+        return "%%s" %% self.string
 
 
 Error.__str__ = _str
@@ -731,7 +728,7 @@ def nbd():
     h.close()
 
 
-";
+|};
 
   List.iter (
     fun { enum_prefix; enums } ->
@@ -769,8 +766,7 @@ def nbd():
       ) ctxts;
   ) metadata_namespaces;
 
-  pr "\
-
+  pr {|
 
 class Buffer(object):
     '''Asynchronous I/O persistent buffer'''
@@ -804,6 +800,14 @@ class Buffer(object):
         '''
         return cls.from_buffer(bytearray(ba))
 
+    def _initialize(self):
+        # Uninitialized buffers remain so for as long as possible.
+        # However the first read or write to an uninitialized buffer
+        # has to initialize it  This is the common code to do that.
+        if not hasattr(self, '_init'):
+            self._o = bytearray(len(self._o))
+            self._init = True
+
     def to_buffer(self):
         '''Return a shared view of the AIO buffer contents.
 
@@ -811,9 +815,7 @@ class Buffer(object):
         visible to nbd.aio_pwrite, and changes from nbd.aio_pread are
         visible in the buffer.
         '''
-        if not hasattr(self, '_init'):
-            self._o = bytearray(len(self._o))
-            self._init = True
+        self._initialize()
         return self._o
 
     def to_bytearray(self):
@@ -834,6 +836,14 @@ class Buffer(object):
         '''Return the size of an AIO buffer.'''
         return len(self._o)
 
+    def __getitem__(self, key):
+        self._initialize()
+        return self._o[key]
+
+    def __setitem__(self, key, newvalue):
+        self._initialize()
+        self._o[key] = newvalue
+
     def is_zero(self, offset=0, size=-1):
         '''Returns true if and only if all bytes in the buffer are zeroes.
 
@@ -867,7 +877,7 @@ class NBD(object):
 
     def _check_not_closed(self):
         if not self._o:
-            raise ClosedHandle(\"libnbd: method called on closed handle\")
+            raise ClosedHandle("libnbd: method called on closed handle")
 
     def close(self):
         '''Explicitly close the NBD handle and underlying connection.
@@ -884,7 +894,7 @@ class NBD(object):
         libnbdmod.close(self._o)
         self._o = None
 
-";
+|};
 
   List.iter (
     fun (name, { args; optargs; shortdesc; longdesc }) ->
@@ -945,13 +955,12 @@ class NBD(object):
   ) handle_calls;
 
   (* For nbdsh. *)
-  pr "\
-
+  pr {|
 package_name = NBD().get_package_name()
 __version__ = NBD().get_version()
 
-if __name__ == \"__main__\":
+if __name__ == "__main__":
     import nbdsh
 
     nbdsh.shell()
-"
+|}
