@@ -491,34 +491,7 @@ let rec print_rust_handle_call_comment name call =
 
 (* Convert POD to rustdoc markdown. *)
 and longdesc_to_markdown name longdesc =
-  (* Replace any POD <> expression *)
-  let content =
-    Str.global_substitute (Str.regexp {|[A-Z]<[^>]+?>|})
-      (fun s ->
-        let expr = Str.matched_string s in
-        let len = String.length expr in
-        let c = expr.[0] and content = String.sub expr 2 (len-3) in
-        match c with
-        | 'C' -> sprintf "`%s`" content (* C<...> becomes `...` *)
-        | 'B' -> sprintf "<b>%s</b>" content
-        | 'I' | 'F' -> sprintf "<i>%s</i>" content
-        | 'E' -> sprintf "&%s;" content
-        | 'L' ->
-           let len = String.length content in
-           if string_starts_with ~prefix:"nbd_" content then (
-             let n = String.sub content 4 (len - 7) in
-             if n <> "get_error" && n <> "get_errno" && n <> "close" then
-               sprintf "[%s](Handle::%s)" n n
-             else
-               sprintf "`%s`" n
-           )
-           else (* external manual page - how to link XXX *)
-             sprintf "<i>%s</i>" content
-        | _ ->
-           failwithf "rust: API documentation for %s contains '%s' which
-                      cannot be converted to Rust markdown" name expr
-      )
-      longdesc in
+  let content = replace_pod_expr name longdesc in
 
   (* Split input into lines for rest of the processing. *)
   let lines = nsplit "\n" content in
@@ -559,6 +532,73 @@ and longdesc_to_markdown name longdesc =
         Some s
   ) lines
 
+(* Replace any POD X<> expressions.
+ * JWZ's rule applies here so we cannot use regexps to parse this.
+ *)
+and replace_pod_expr name s =
+  (* Split string s into the part before "X<", the operator 'X', and
+   * the remainder of the string.
+   *)
+  let before, op, after =
+    try
+      let i = Str.search_forward (Str.regexp {|[A-Z]<|}) s 0 in
+      let op = Some s.[i] in
+      let before = String.sub s 0 i in
+      let after = String.sub s (i+2) (String.length s - i - 2) in
+      before, op, after
+    with Not_found -> s, None, "" in
+
+  match op with
+  | None -> before
+  | Some op ->
+     (* Find the content inside the X<...> operator. *)
+     let content, rest =
+       let n = String.length after in
+       let rec loop i depth =
+         if i > n then
+           failwithf "rust: API documentation for %s contains unclosed %c<...>"
+             name op;
+         let c = after.[i] in
+         match c, depth with
+         | '<', _ -> loop (i+1) (depth+1)
+         | '>', 0 -> i
+         | '>', _ -> loop (i+1) (depth-1)
+         | _ -> loop (i+1) depth
+       in
+       let i = loop 0 0 in
+       String.sub after 0 i, String.sub after (i+1) (n-i-1) in
+
+     (* Process the operator. *)
+     let new_content =
+       let content = replace_pod_expr name content in
+       match op with
+       | 'C' -> sprintf "`%s`" content (* C<...> becomes `...` *)
+       | 'B' -> sprintf "<b>%s</b>" content
+       | 'I' | 'F' -> sprintf "<i>%s</i>" content
+       | 'E' -> sprintf "&%s;" content
+       | 'S' -> (* non-breaking spaces *)
+          Str.global_replace (Str.regexp {| \|\t|}) "&nbsp;" content
+       | 'L' ->
+          let len = String.length content in
+          if string_starts_with ~prefix:"nbd_" content then (
+            let n = String.sub content 4 (len - 7) in
+            if n <> "get_error" && n <> "get_errno" && n <> "close" then
+              sprintf "[%s](Handle::%s)" n n
+            else
+              sprintf "`%s`" n
+          )
+          else if string_starts_with ~prefix:"http://" content ||
+                    string_starts_with ~prefix:"https://" content then
+            sprintf "[%s](%s)" content content
+          else (* external manual page - how to link XXX *)
+            sprintf "<i>%s</i>" content
+       | _ ->
+          failwithf "rust: API documentation for %s contains '%c<...>' which
+                     cannot be converted to Rust markdown" name op in
+
+     (* Assemble the output. *)
+     before ^ new_content ^ replace_pod_expr name rest
+
 (* Print a Rust expression which converts Rust like arguments to FFI like
    arguments, makes a call on the raw FFI handle, and converts the return
    value to a Rusty type. The expression assumes that variables with name
