@@ -274,8 +274,7 @@ let generate_docs_nbd_pod name { args; optargs; ret;
     ) links;
     pr ".\n\n" in
 
-  pr "\
-=head1 AUTHORS
+  pr {|=head1 AUTHORS
 
 Eric Blake
 
@@ -284,4 +283,4 @@ Richard W.M. Jones
 =head1 COPYRIGHT
 
 Copyright Red Hat
-"
+|}
