@@ -39,6 +39,7 @@
 
 #include "array-size.h"
 #include "const-string-vector.h"
+#include "parse-bool.h"
 #include "vector.h"
 
 #include "internal.h"
@@ -161,29 +162,17 @@ error:
   return -1;
 }
 
-/* Similar to nbdkit_parse_bool */
 static int
-parse_bool (const char *param, const char *value)
+parse_bool_param (const char *param, const char *value)
 {
-  if (!strcmp (value, "1") ||
-      !strcasecmp (value, "true") ||
-      !strcasecmp (value, "t") ||
-      !strcasecmp (value, "yes") ||
-      !strcasecmp (value, "y") ||
-      !strcasecmp (value, "on"))
-    return 1;
-
-  if (!strcmp (value, "0") ||
-      !strcasecmp (value, "false") ||
-      !strcasecmp (value, "f") ||
-      !strcasecmp (value, "no") ||
-      !strcasecmp (value, "n") ||
-      !strcasecmp (value, "off"))
-    return 0;
-
-  set_error (EINVAL, "could not parse %s parameter, expecting %s=true|false",
-             param, param);
-  return -1;
+  int r = parse_bool (value);
+  if (r == -1) {
+    set_error (EINVAL, "could not parse %s parameter, "
+               "expecting %s=true|false",
+               param, param);
+    return -1;
+  }
+  return r;
 }
 
 static int
@@ -302,8 +291,6 @@ uri_connect_ssh (struct nbd_handle *h, const xmlURIPtr uri,
   const_string_vector_append (&ssh_command, "-p");
   const_string_vector_append (&ssh_command, port_str);
 
-  const_string_vector_append (&ssh_command, uri->server);
-
   /* SSH user always comes from the authority part of the URI. */
   if (uri->user) {
     /* Don't allow any special tokens in the username which ssh might
@@ -321,6 +308,10 @@ uri_connect_ssh (struct nbd_handle *h, const xmlURIPtr uri,
     const_string_vector_append (&ssh_command, ssh_user);
   }
 
+  const_string_vector_append (&ssh_command, "--");
+  const_string_vector_append (&ssh_command, uri->server);
+
+  /* All further argc beginning with - are options to nc. */
   const_string_vector_append (&ssh_command, "nc");
   if (unixsocket) {
     const_string_vector_append (&ssh_command, "-U");
@@ -499,7 +490,7 @@ nbd_unlocked_aio_connect_uri (struct nbd_handle *h, const char *raw_uri)
       nbd_port = queries.ptr[i].value;
     }
     else if (strcasecmp (queries.ptr[i].name, "compress") == 0) {
-      int v = parse_bool ("compress", queries.ptr[i].value);
+      int v = parse_bool_param ("compress", queries.ptr[i].value);
       if (v == -1)
         goto cleanup;
       ssh_compress = v;
@@ -532,7 +523,7 @@ nbd_unlocked_aio_connect_uri (struct nbd_handle *h, const char *raw_uri)
       tls_username = queries.ptr[i].value; /* set below */
     }
     else if (strcasecmp (queries.ptr[i].name, "tls-verify-peer") == 0) {
-      int v = parse_bool ("tls-verify-peer", queries.ptr[i].value);
+      int v = parse_bool_param ("tls-verify-peer", queries.ptr[i].value);
       if (v == -1)
         goto cleanup;
       if (nbd_unlocked_set_tls_verify_peer (h, v) == -1)
