@@ -64,8 +64,9 @@ main (int argc, char *argv[])
   struct nbd_handle *nbd;
   const char *cmd[] = {
     NBDKIT, "-s", "-v", "--exit-with-parent",
-    "memory", "1024",
+    "memory", "1025",
     "--filter=blocksize-policy", "blocksize-minimum=512",
+    "blocksize-error-policy=allow",
     NULL
   };
   uint32_t strict;
@@ -92,14 +93,29 @@ main (int argc, char *argv[])
     fprintf (stderr, "%s\n", nbd_get_error ());
     exit (EXIT_FAILURE);
   }
-  if (nbd_aio_pread (nbd, buf, 1, 1, NBD_NULL_COMPLETION, 0) != -1) {
+  if (nbd_aio_pread (nbd, buf, 1, 512, NBD_NULL_COMPLETION, 0) != -1) {
     fprintf (stderr, "%s: test failed: "
-             "unaligned nbd_aio_pread did not fail\n",
+             "unaligned nbd_aio_pread count did not fail\n",
+             argv[0]);
+    exit (EXIT_FAILURE);
+  }
+  check (EINVAL, "nbd_aio_pread: ");
+  if (nbd_aio_pread (nbd, buf, 512, 1, NBD_NULL_COMPLETION, 0) != -1) {
+    fprintf (stderr, "%s: test failed: "
+             "unaligned nbd_aio_pread offset did not fail\n",
              argv[0]);
     exit (EXIT_FAILURE);
   }
   check (EINVAL, "nbd_aio_pread: ");
 
+  /* However, access to unaligned tail is permitted */
+  if (nbd_pread (nbd, buf, 1, 1024, 0) != 0) {
+    fprintf (stderr, "%s: test failed: "
+             "nbd_pread to unaligned tail unexpected failure: %s\n",
+             argv[0], nbd_get_error ());
+    exit (EXIT_FAILURE);
+  }
+
   nbd_close (nbd);
   exit (EXIT_SUCCESS);
 }
